if (!com) var com = {};
if (!com.wordstream) com.wordstream = {};
if (!com.wordstream.plugin) com.wordstream.plugin = {};

com.wordstream.plugin.IFrameHandler = function(iframe)
{
    this._nonWhiteSpace = /[^\s\-_.!,?:;\\/"]/;
    this._whiteSpace = /[\s\-_.!,?:;\\/"]/;
    this._tagName = /[a-zA-Z]+/

    this.phraseBreakTags = {'br': true, 'td': true, 'p': true};
    this.globalSpaceDetector = false;

    this.mainTextArea = null;
    this.contentIFrame = iframe;
    this.IFrameDoc = this.contentIFrame.contentWindow.document;

    this.HTMLContent = this.IFrameDoc.body;
    this.plainText = "";

    this._getPlainText = function()
    {
        function recurse(elem, target)
        {
            if (elem instanceof HTMLParagraphElement || elem instanceof HTMLBRElement)
                this.globalSpaceDetector = true;
            var children = elem.childNodes;
            for (var i = 0; i < children.length; i++)
            {
                if (children[i].nodeValue != null)
                {
                    if (this.globalSpaceDetector == true)
                    {
                        target.push ('.');
                        this.globalSpaceDetector = false;
                    }
                    target.push(children[i].nodeValue);
                }
                recurse(children[i], target);
            }
        }

        var tar = [];

        recurse(this.HTMLContent, tar);
        return tar.join('');
    }

    this.deHighlight = function()
    {
        // De-tag the content of our highlight tags
        var deTaggedList = [];
        var newHTMLContent = '' + this.HTMLContent.innerHTML;
        var openingTag = '<span role="wordstream-highlights" style="background-color: rgb(250, 238, 85);">';
        var closingTag = '</span>';
        var prevPos = 0;

        var watchPos = 0;

        while(1)
        {
            var opener = newHTMLContent.indexOf(openingTag, prevPos);
            if (opener != -1)
            {
                deTaggedList.push(newHTMLContent.substr(prevPos, opener-prevPos));
                var closer = newHTMLContent.indexOf(closingTag, opener + openingTag.length);
                deTaggedList.push(newHTMLContent.substr(opener+openingTag.length, closer-(opener+openingTag.length)));
                prevPos = closer + closingTag.length;
            }
            else
            {
                deTaggedList.push(newHTMLContent.substr(prevPos));
                break;
            }
        }

        this.HTMLContent.innerHTML = deTaggedList.join('');
    }

    this.highlightUsedWords = function(highlightMap)
    {
        // De-tag the content of our highlight tags
        var deTaggedList = [];
        var newHTMLContent = '' + this.HTMLContent.innerHTML;
        var openingTag = '<span role="wordstream-highlights" style="background-color: rgb(250, 238, 85);">';
        var closingTag = '</span>';
        var prevPos = 0;

        var watchPos = 0;

        this.deHighlight();

        var origHTML = '<f>' + this.HTMLContent.innerHTML + '</f>';
        var html = '<f>' + this.HTMLContent.innerHTML + '</f>';

        // Replace with exactly 6 spaces so we can preserve offsets
        html = html.replace(/\&....;/g, "      ");
        html = html.replace(/\&..;/g,  "    ");

        var wc = 0;
        var preInc = false;
        var prevBumped = true;
        var prevOffset = 0;
        var newContent = [];

        var offset = html.indexOf('<', prevOffset);
        if (offset > 0)
        {
            newContent.push(origHTML.substr(0, offset));
        }

        while(offset != -1)
        {
            // we're in a tag, get us outta here
            var tagEnd = html.indexOf('>', offset);
            var tag = html.substr(offset, tagEnd+1);
            var tagName = tag.match(this._tagName)[0].toLowerCase();

            if (tagName in this.phraseBreakTags)
            {
                preInc = true;
            }

            // Next tag
            var next = html.indexOf('<', tagEnd);
            var endOffset = next;
            if (endOffset == -1)
                endOffset = html.length - 1;

            newContent.push(html.substr(offset, tagEnd+1 - offset));
            if (endOffset - tagEnd > 1)
            {
                // This is the meat of the nodeText within 2 tags (it has to be)
                // We're not in a tag so get all the words up until the next tag

                // Does this begin with a delimiting character
                var begin = html[tagEnd+1];
                var ws = begin.search(this._whiteSpace);
                if (ws != -1) preInc = true;

                if (prevBumped == false && preInc == true)
                {
                    preInc = false;
                    prevBumped = true;
                    wc++;
                }

                var innerTextOffset = tagEnd+1;
                var lastCopied = innerTextOffset;
                var innerText = html.substr(tagEnd+1, endOffset - (tagEnd+1));
                // parse out the words from this business
                var begin = innerText.search(this._nonWhiteSpace);
                while (begin != -1)
                {
                    innerTextOffset += begin;
                    innerText = innerText.substr(begin);

                    var innerTextEnd = innerText.search(this._whiteSpace);
                    if (innerTextEnd != -1)
                    {
                        // We have a whole word
                        if (watchPos < highlightMap.length)
                        {
                            if (highlightMap[watchPos] < wc)
                                watchPos ++;

                            if (watchPos < highlightMap.length)
                            {
                                if (highlightMap[watchPos] == wc)
                                {
                                    newContent.push(origHTML.substr(lastCopied, innerTextOffset - lastCopied));
                                    newContent.push(openingTag);
                                    newContent.push(origHTML.substr(innerTextOffset, innerTextEnd));
                                    newContent.push(closingTag);

                                    lastCopied = innerTextOffset + innerTextEnd;
                                    watchPos ++;
                                }
                            }
                        }

                        innerText = innerText.substr(innerTextEnd);
                        innerTextOffset += innerTextEnd;
                        begin = innerText.search(this._nonWhiteSpace);
                        prevBumped = true;

                        wc++;
                    }
                    else
                    {
                        innerTextEnd = innerText.length;
                        // We have a potential partial word
                        if (watchPos < highlightMap.length)
                        {
                            if (highlightMap[watchPos] < wc)
                                watchPos ++;

                            if (watchPos < highlightMap.length)
                            {
                                if (highlightMap[watchPos] == wc)
                                {
                                    newContent.push(origHTML.substr(lastCopied, innerTextOffset - lastCopied));
                                    newContent.push(openingTag);
                                    newContent.push(origHTML.substr(innerTextOffset, innerTextEnd));
                                    newContent.push(closingTag);

                                    lastCopied = innerTextOffset + innerTextEnd;
                                }
                            }
                        }

                        prevBumped = false;
                        break;
                    }
                }

                if (lastCopied < endOffset)
                {
                    newContent.push(origHTML.substr(lastCopied, endOffset - lastCopied));
                }
            }

            offset = next;
        }

        var repl = newContent.join('');

        this.HTMLContent.innerHTML = repl.substr(3, repl.length - 7);
    }

    this.rescan = function()
    {
        var newText = this._getPlainText();
        this.plainText = newText;
        return this.plainText;
    }
}

