if (!com) var com = {};
if (!com.wordstream) com.wordstream = {};
if (!com.wordstream.plugin) com.wordstream.plugin = {};

com.wordstream.plugin.PhraseIterator = function (wordSmith, sortDesc)
{
    wordSmith._finalize();
    this.wordSmith = wordSmith;
    this.currentPhrase = 0;
    this.sortedPhrases = [];
    var i = 0;

    for each (phrase in this.wordSmith.phraseList)
    {
        if (phrase[4] == true && !(i in this.wordSmith.deletedPhraseIds))
        {
            this.sortedPhrases.push([i, phrase[0], phrase[1], phrase[2]]);
        }
        i++;
    }
    if (this.sortedPhrases.length == 0) return;

    this.sortedPhrases.sort(function(a, b) { return a[2] < b[2] });
    if (!sortDesc) this.sortedPhrases.reverse();
}

com.wordstream.plugin.PhraseIterator.prototype.next = function()
{
    if (this.currentPhrase >= this.sortedPhrases.length)
        return false;
    var phraseObj = [this.sortedPhrases[this.currentPhrase][0], this.sortedPhrases[this.currentPhrase][1], (this.sortedPhrases[this.currentPhrase][2] / this.wordSmith.upperLimit) * 100.0,this.sortedPhrases[this.currentPhrase][3]];
    this.currentPhrase ++;
    return phraseObj;
}

com.wordstream.plugin.PhraseByWordIterator = function(wordSmith, word, sortDesc)
{
    wordSmith._finalize();
    this.wordSmith = wordSmith;
    this.currentPhrase = 0;
    this.word = word;

    this.pbw = null;
    if (wordSmith.query != null)
    {
        this.pbw = this.wordSmith.filteredPhrasesByWord;
    }
    else
    {
        this.pbw = this.wordSmith.phrasesByWord;
    }

    // Sort by relative volume
    this.pbw[this.word].sort(function(a, b) { return a[1] < b[1] });
    if (!sortDesc) this.pbw[this.word].reverse();
}

com.wordstream.plugin.PhraseByWordIterator.prototype.next = function()
{
    if (this.currentPhrase >= this.pbw[this.word].length)
        return false;

    while(1)
    {
        var phrase = this.pbw[this.word][this.currentPhrase];
        var phraseId = phrase[0];

        if (!(phraseId in this.wordSmith.deletedPhraseIds))
        {
            var phraseObj = [phraseId, this.wordSmith.phraseList[phraseId][0], (phrase[1] / this.wordSmith.upperLimit) * 100.0];
            this.currentPhrase ++;
            break;
        }
        else
        {
            this.currentPhrase ++;
            if (this.currentPhrase >= this.pbw[this.word].length)
                return false;
        }
    }
    return phraseObj;
}

com.wordstream.plugin.WordIterator = function(wordSmith, sortDesc)
{
    wordSmith._finalize();
    this.wordSmith = wordSmith;
    this.currentWord = 0;

    this.wl = null;
    this.pc = null;
    if (wordSmith.query != null)
    {
        this.wl = this.wordSmith.filteredWordList;
        this.pc = this.wordSmith.filteredPhrasesByWord;
        this.dc = this.wordSmith.filteredDeletedPhrasesByWord;
    }
    else
    {
        this.wl = this.wordSmith.wordList;
        this.pc = this.wordSmith.phrasesByWord;
        this.dc = this.wordSmith.deletedPhrasesByWord;
    }

    if (this.wl.length == 0) return;

    this.wl.sort(function(a,b){ return a[1] < b[1] });
    this.biggestWord = this.wl[0][1];
    if (!sortDesc) this.wl.reverse();
}

com.wordstream.plugin.WordIterator.prototype.next = function()
{
    if (this.currentWord >= this.wl.length)
        return false;

    while(1)
    {
        var word = this.wl[this.currentWord];
        if (!this.wordSmith.deletedWords.hasOwnProperty(word[0]))
        {
            var reduction = 0;
            if (this.dc.hasOwnProperty(word[0]))
            {
                reduction = this.dc[word[0]];
            }
            var total = this.pc[word[0]].length - reduction;
            if (total <= 0)
            {
                this.currentWord ++;
                if (this.currentWord >= this.wl.length)
                    return false;
                break;
            }
            var wordObj = [this.wordSmith.wordIdMap[word[0]], word[0], (word[1] / this.biggestWord) * 100.0, word[1]];
            this.currentWord ++;
            break;
        }
        else
        {
            this.currentWord ++;
            if (this.currentWord >= this.wl.length)
                return false;
        }

    }
    return wordObj;
}

com.wordstream.plugin.WordSmith = function(topicWords)
{
    this._stopWords = {'a': true, 'an': true, 'are': true, 'as': true, 'at': true, 'be': true, 'by': true, 'de': true,
                  'en': true, 'for': true, 'from': true, 'how': true, 'i': true, 'in': true, 'is': true, 'it': true,
                  'la': true, 'of': true, 'on': true, 'or': true, 'that': true, 'the': true, 'this': true, 'to': true,
                  'und': true, 'was': true, 'what': true, 'when': true, 'where': true, 'who': true, 'will': true,
                  'with': true, 'with': true, 'or': true, 'and': true};
    this._questionWords = {'who': true, 'whom': true, 'whose': true, 'where': true, 'what': true, 'when': true, 'why': true,
                     'is': true, 'does': true, 'do': true, 'can': true, "can't": true, 'are': true, "aren't": true,
                     'was': true, 'which': true, "don't": true, 'should': true, "shouldn't": true, 'could':true,
                     "couldn't": true, 'will': true, "won't": true, 'did': true, 'versus': true, 'vs': true,
                     'difference': true, 'bigger': true, 'smaller': true, 'best': true, 'biggest': true, 'worst': true,
                     'cheapest': true, 'difference': true, 'popular': true};
    this._PHRASE_CAP = 200;
    this._wordSplitter = /[\s\-_.!,?"<>]+/;
    this._phraseSplitter = /[.,!?:;\\/\n]+/;
    this._laxedPhraseSplitter = /[.,!?:\\/;]+/;
    this.uniquePhraseId = 0;
    this.uniqueWordId = 0;
    this.phraseList = [];
    this.phraseWordSequence = [];
    this.phrasesByWord = {};
    this.wordList = [];
    this.wordIdMap = {};
    this.phraseWordSequenceMap = {};
    this.phraseCountById = {};
    this.finalized = false;
    this.usedQuestionPhrases = 0;
    this.usedPhrases = 0;
    this.usedWords = 0;
    this.query = null;
    this.queryCount = 0;
    this.filteredPhrasesByWord = {};
    this.filteredWordList = [];
    this.deletedWords = {};
    this.deletedPhraseIds = {};
    this.deletedPhrasesByWord = {};
    this.filteredDeletedPhraseIds = {};
    this.filteredDeletedPhrasesByWord = {};
    this.topicWords = topicWords;
    this.upperLimit = 0;
    this.allWords = {};

    this.setUpperLimit = function(lim)
    {
        this.upperLimit = lim;
    }

    this.setFilter = function(query)
    {
        this.filteredDeletedPhraseIds = {};
        this.filteredDeletedPhrasesByWord = {};
        this.queryCount = 0;
        words = query.split(/[\s]+/);
        if (words.length > 0)
            if (words[0].length == 0)
                words.splice(0, 1);
        if (words.length > 0)
            if (words[words.length - 1].length == 0)
                words.splice(-1, 1);

        if (words.length == 0)
        {
            this.query = null;
        }
        else
        {
            this.query = {};
            for (var i = 0; i < words.length; i++)
            {
                if (!this.query.hasOwnProperty(words))
                {
                    this.query[words[i]] = true;
                    this.queryCount++;
                }
            }
        }

        if (this.query == null)
        {
            for (var i = 0; i < this.phraseList.length; i++)
            {
                this.phraseList[i][4] = true;
            }
            return;
        }

        this.filteredWordList = [];
        this.filteredPhrasesByWord = {};
        for (var i = 0; i < this.phraseList.length; i++)
        {
            var phrase = this.phraseList[i];
            phrase[4] = false;
            var seenWords = {};
            var count = 0;
            var words = phrase[3];
            for (var j = 0; j < words.length; j++)
            {
                var word = words[j];
                phrase[4] = false;
                if (this.query.hasOwnProperty(word))
                {
                    if (!seenWords.hasOwnProperty(word))
                    {
                        count ++;
                        if (count == this.queryCount)
                        {
                            phrase[4] = true;
                            var deleted = false;
                            if (i in this.deletedPhraseIds)
                            {
                                this.filteredDeletedPhraseIds[i] = true;
                                deleted = true;
                            }

                            var newSeenWords = {};
                            for (var k = 0; k < words.length; k++)
                            {
                                var newWord = words[k];
                                if (deleted)
                                {
                                    if (this.filteredDeletedPhrasesByWord.hasOwnProperty(newWord))
                                    {
                                        this.filteredDeletedPhrasesByWord[newWord] ++;
                                    }
                                    else
                                    {
                                        this.filteredDeletedPhrasesByWord[newWord] = 1;
                                    }
                                }

                                if (this.filteredPhrasesByWord.hasOwnProperty(newWord))
                                {
                                    if (!newSeenWords.hasOwnProperty(newWord))
                                    {
                                        this.filteredPhrasesByWord[newWord].push([i, phrase[1]]);
                                    }
                                }
                                else
                                {
                                    this.filteredPhrasesByWord[newWord] = [[i, phrase[1]]];
                                    if (!this._stopWords.hasOwnProperty(newWord) && !this.topicWords.hasOwnProperty(newWord) && (newWord.length > 1)) this.filteredWordList.push([newWord]);
                                }
                                newSeenWords[newWord] = true;
                            }
                            break;
                        }
                    }

                }
                seenWords[word] = true;
            }
        }

        for each (word in this.filteredWordList)
        {
            word.push(this.filteredPhrasesByWord[word[0]].length);
        }
    }

    this.add = function(keyword, relativeVolume)
    {
        if (this.phraseList.length >= this._PHRASE_CAP) return;

        var isQuestion = false;
        var words = keyword.split(this._wordSplitter);

        if (words.length > 0)
            if (words[0].length == 0)
                words.splice(0, 1);
        if (words.length > 0)
            if (words[words.length - 1].length == 0)
                words.splice(-1, 1);
        if (words.length == 0) return;

        for (i = 0; i < words.length; i++)
        {
            if (this._questionWords.hasOwnProperty(words[i]))
            {
                isQuestion = true;
                break;
            }
        }

        var pws = [];
        var phraseId = this.uniquePhraseId;

        var wouldBeIds = [];
        var tempUnique = this.uniqueWordId;
        for each (word in words)
        {
            if (!this.wordIdMap.hasOwnProperty(word))
            {
                wordId = tempUnique++;
            }
            else
            {
                wordId = this.wordIdMap[word];
            }
            wouldBeIds.push(wordId);
        }
        if (this.phraseWordSequenceMap.hasOwnProperty(wouldBeIds.join('_')))
            return;

        for each (word in words)
        {
            var newWord = false;
            var wordId = 0;
            if (!this.wordIdMap.hasOwnProperty(word))
            {
                newWord = true;
                wordId = this.uniqueWordId++;
                this.wordIdMap[word] = wordId;
            }
            else
            {
                wordId = this.wordIdMap[word];
            }

            // Categorize the word regardless of it being a stop-word
            pws.push(wordId);
            if (newWord && !this._stopWords.hasOwnProperty(word) && !this.topicWords.hasOwnProperty(word) && (word.length > 1))
            {
                this.wordList.push([word]);
                this.allWords[word] = 0;
            }

            if (!this.phrasesByWord.hasOwnProperty(word))
            {
                this.phrasesByWord[word] = [[phraseId, relativeVolume]];
            }
            else
            {
                this.phrasesByWord[word].push([phraseId, relativeVolume]);
            }
        }

        if (relativeVolume > this.upperLimit) this.upperLimit = relativeVolume;
        this.phraseList.push([keyword, relativeVolume, isQuestion, words, true]);
        this.phraseWordSequence.push(pws);
        this.phraseCountById[phraseId] = 0;
        this.phraseWordSequenceMap[pws.join('_')] = phraseId;
        this.uniquePhraseId++;
    }

    this._finalize = function()
    {
        if (this.finalized) return;
        for each (word in this.wordList)
        {
            word.push(this.phrasesByWord[word[0]].length);
        }
        this.finalized = true;
    }

    this.phrases = function(sortDesc)
    {
        return new com.wordstream.plugin.PhraseIterator(this, sortDesc);
    }

    this.words = function(sortDesc)
    {
        return new com.wordstream.plugin.WordIterator(this, sortDesc);
    }

    this.listPhrasesByWord = function(word, sortDesc)
    {
        return new com.wordstream.plugin.PhraseByWordIterator(this, word, sortDesc);
    }

    this.mapPhrase = function(words, mode)
    {
        var lowerBound = 0;
        var upperBound = -1;
        var phraseMap = [];
        if (mode == 0)
        {
            for (var i = 0; i < words.length; i++)
            {
                var word = words[i].toLowerCase();
                if (this.wordIdMap.hasOwnProperty(word))
                {
                    phraseMap.push(this.wordIdMap[word]);
                    upperBound = i;
                }
                else
                {
                    if (upperBound == -1)
                        lowerBound = i;
                    phraseMap.push(-1);
                }
            }
        }

        if (mode == 1)
        {
            for (var i = 0; i < words.length; i++)
            {
                var word = words[i].toLowerCase();
                if (this.allWords.hasOwnProperty(word))
                {
                    phraseMap.push(this.wordIdMap[word]);
                    upperBound = i;
                }
                else
                {
                    if (upperBound == -1)
                        lowerBound = i;
                    phraseMap.push(-1);
                }
            }
        }

        // Upper bound allows rejection on entire mappings
        return [lowerBound, upperBound, phraseMap];
    }

    this.getPhrase = function(key)
    {
        if (this.phraseWordSequenceMap.hasOwnProperty(key))
        {
            return this.phraseWordSequenceMap[key];
        }
        else
        {
            return -1;
        }
    }

    this.analyzeDocument = function(docText, phraseBreaker, mode, highlight)
    {
        // Replace all &nbsp; with a space character to ease counting
        for (word in this.allWords)
        {
            this.allWords[word] = 0;
        }
        docText = docText.replace(/\&nbsp;/g, " ");

        var wc=0;
        var highlightWords = [];
        var foundPhrases = 0;
        for (var p in this.phraseCountById)
        {
            this.phraseCountById[p] = 0;
        }
        var phrases = docText.split(phraseBreaker);
        if (phrases.length > 0)
        {
            if (phrases[0].length == 0)
                phrases.splice(0,1);
        }
        if (phrases.length > 0)
        {
            if (phrases[phrases.length-1].length == 0)
                phrases.splice(-1,1);
        }

        var prevCount = 0;
        for each (phrase in phrases)
        {
            wc += prevCount;
            if (phrase.length <= 1) continue;
            var words = phrase.split(this._wordSplitter);
            // Remove leading/trailing garbage from array
            if (words.length > 0)
            {
                if (words[0].length == 0)
                    words.splice(0,1);
            }
            if (words.length > 0)
            {
                if (words[words.length-1].length == 0)
                    words.splice(-1,1);
            }

            prevCount = words.length;

            if (words.length == 0) continue;
            for (var i = 0; i < words.length; i++)
            {
                var word = words[i].toLowerCase();
                if (this.allWords.hasOwnProperty(word))
                {
                    this.allWords[word] ++;
                }
            }

            // Do regular assessment
            var mapping = this.mapPhrase(words, 0);
            if (mode == 0 || mode == 2)
                if (mapping[1] == -1) continue;
            if (mapping[1] != -1)
            {
                var highlightMap = new Array(words.length);
                for (var a = mapping[0]; a <= mapping[1]; a++)
                {
                    for (var b = a+1; b <= mapping[1]+1; b++)
                    {
                        var key = mapping[2].slice(a, b).join('_');
                        var phraseId = this.getPhrase(key);
                        if (phraseId != -1)
                        {
                            foundPhrases++;
                            this.phraseCountById[phraseId]++;

                            if (highlight)
                            {
                                if (mode == 2 && this.phraseList[phraseId][2] == false) continue;
                                for (var c = a; c < b; c++)
                                {
                                    highlightMap[c] = true;
                                }
                            }
                        }
                    }
                }
            }

            if (highlight)
            {
                if (mode == 1)
                {
                    // This is just for highlighting
                    var highlightMap = new Array(words.length);
                    for (var a = 0; a < words.length; a++)
                    {
                        var word = words[a].toLowerCase();
                        if (this.allWords.hasOwnProperty(word))
                            highlightMap[a] = true;
                    }
                }

                for (var a = 0; a < highlightMap.length; a++)
                {
                    if (highlightMap[a] == true)
                    {
                        highlightWords.push(wc+a);
                    }
                }
            }
        }

        return highlightWords;
    }

    this.deleteWord = function(word)
    {
        var deletedPhrases = {};

        if (!this.deletedWords.hasOwnProperty(word))
        {
            this.deletedWords[word] = true;

            if (this.query != null)
            {
                for (var i = 0; i < this.filteredPhrasesByWord[word].length; i++)
                {
                    if (!this.filteredDeletedPhraseIds.hasOwnProperty(this.filteredPhrasesByWord[word][i][0]))
                    {
                        this.filteredDeletedPhraseIds[this.filteredPhrasesByWord[word][i][0]] = true;
                    }
                }
            }

            for (var i = 0; i < this.phrasesByWord[word].length; i++)
            {
                if (!this.deletedPhraseIds.hasOwnProperty(this.phrasesByWord[word][i][0]))
                {
                    this.deletedPhraseIds[this.phrasesByWord[word][i][0]] = true;
                }
                deletedPhrases[this.phrasesByWord[word][i][0]] = true;
            }
        }

        return deletedPhrases;
    }

    this.deletePhrase = function(phraseId)
    {
        if (!(phraseId in this.deletedPhraseIds))
        {
            var words = this.phraseList[phraseId][3];

            for (var i = 0; i < words.length; i++)
            {
                var word = words[i];
                if (this.filteredDeletedPhrasesByWord.hasOwnProperty(word))
                {
                    this.filteredDeletedPhrasesByWord[word]++;
                }
                else
                {
                    this.filteredDeletedPhrasesByWord[word] = 1;
                }
            }

            for (var i = 0; i < words.length; i++)
            {
                var word = words[i];

                if (this.deletedPhrasesByWord.hasOwnProperty(word))
                {
                    this.deletedPhrasesByWord[word]++;
                }
                else
                {
                    this.deletedPhrasesByWord[word] = 1;
                }
                if (this.phrasesByWord[word].length <= this.deletedPhrasesByWord[word])
                {
                    this.deletedWords[word] = true;
                }
            }

            this.deletedPhraseIds[phraseId] = true;
        }
    }
}